#include <avr/io.h>
#include <avr/interrupt.h>
#include <portbits.h>
#include "global.h"
#include "uart.h"

#define XTAL_CPU        24000000
#define UART_BAUD_RATE    500000

volatile unsigned short spalte;
volatile unsigned char sync,color;

#define GETRES		1

unsigned short resx, resy;
unsigned short hsync, vsync, measdone;

void measure_sig(void);

#define HSYNC	PINB_0
#define VSYNC	PIND_2

int main(void)
{
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE, XTAL_CPU));

	DIDR0=255;
	PORTB=255;
	PORTD=255;

	ADMUX=(1<<REFS0)|(1<<REFS1)|(1<<ADLAR);			// ADC0, 1,1V Ref
	ADCSRA=(1<<ADEN)|(1<<ADATE)|5;					// ADC enable, 750kHz ADC clock
	ADCSRB=5;										// Output Compare 1B

	EICRA=(1<<ISC01)|(1<<ISC00);					// fallende Ecke von HSync

	sei();

	for (;;)
	{	unsigned char c;
		c=uart_getchar();
		if (c==GETRES)
		{	CONVERTW val;
			measure_sig();
			val.w=resx;
			uart_putc(val.b[0]);
			uart_putc(val.b[1]);
			val.w=resy;
			uart_putc(val.b[0]);
			uart_putc(val.b[1]);
			uart_putc(hsync|(vsync*16));
		
		}
		else if (c==65)
		{	spalte=0;
			color=0;
			sync=0;
			TCNT1=0;
			TCCR1A=0;
			OCR1B=0;
			ADMUX=(1<<REFS0)|(1<<REFS1)|(1<<ADLAR);			// ADC0, 1,1V Ref
			if (vsync)
				EICRA=(1<<ISC01)|(1<<ISC00);
			else
				EICRA=(1<<ISC01);
			if (hsync)
				TCCR1B=(1<<ICNC1)|(1<<ICES1)|1;
			else
				TCCR1B=(1<<ICNC1)|1;

			EIMSK=(1<<INT0);
			TIMSK1=(1<<ICIE1);
			sync=0;
			while (!sync);
			sync=0;
			TIFR1=(1<<OCF1B);
			ADCSRA=(1<<ADEN)|(1<<ADATE)|(1<<ADIF)|5;	 // clear ADIF
			while (spalte<resx-32)
            {	while (!(ADCSRA&(1<<ADIF)));				// warten bis Messung ferig
				unsigned char adcv;
				adcv=ADCH;
				ADCSRA=(1<<ADEN)|(1<<ADATE)|(1<<ADIF)|5;	 // clear ADIF
				TIFR1=1<<OCF1B;
				if (sync)
				{	sync=0;
					adcv=255;
				}
				else if (adcv==255)
					adcv=254;
				UDR0=adcv;
            }
			EIMSK=0;
			TCCR1B=0;
			TIMSK1=0;
			uart_putc(0);
			uart_putc(0);
			uart_putc(0);
			uart_putc(0);
			uart_putc(0);
			uart_putc(0);
		}
	}
}

void measure_sig(void)
{	unsigned short countl, counth;
	TIMSK1=0;
	EIMSK=0;
	EICRA=(1<<ISC01);
	TCCR1A=0;
	TCCR1B=(1<<ICNC1)|1;			// HSync Polung
	countl=0;
	counth=0;
	TIFR1=(1<<ICF1);
	while (!(TIFR1&(1<<ICF1)));
	TIFR1=(1<<ICF1);
	while (!(TIFR1&(1<<ICF1)))
	{	if (HSYNC)
			counth++;
		else
			countl++;
	}
	TIFR1=(1<<ICF1);
	if (counth<countl)
		hsync=1;
	else
		hsync=0;

	if (hsync)
		TCCR1B=(1<<ICF1)|(1<<ICES1)|1;
	TIFR1=(1<<ICF1);
	while (!(TIFR1&(1<<ICF1)));
	TIFR1=(1<<ICF1);
	countl=ICR1;
	while (!(TIFR1&(1<<ICF1)));
	TIFR1=(1<<ICF1);
	counth=ICR1;
	resx=counth-countl;

	EIFR=(1<<ISC01);
	countl=0;
	counth=0;
	EIFR=(1<<INTF0);
	while (!(EIFR&(1<<INTF0)));
	EIFR=(1<<INTF0);
	while (!(EIFR&(1<<INTF0)))
	{	while (!(TIFR1&(1<<ICF1)));
		TIFR1=(1<<ICF1);
		if (VSYNC)
			counth++;
		else
			countl++;
	}
	EIFR=(1<<INTF0);
	if (counth<countl)
		vsync=1;
	else
		vsync=0;

	if (vsync)
		EICRA=(1<<ISC01)|(1<<ISC00);
	countl=0;
	counth=0;
	EIFR=(1<<INTF0);
	TIFR1=(1<<ICF1);
	while (!(EIFR&(1<<INTF0)));
	while (!(TIFR1&(1<<ICF1)));
	TIFR1=(1<<ICF1);
	EIFR=(1<<INTF0);
	while (!(EIFR&(1<<INTF0)))
	{	while (!(TIFR1&(1<<ICF1)));
		TIFR1=(1<<ICF1);
		countl++;
	}
	EIFR=(1<<INTF0);
	resy=countl;

	measdone=1;
}

ISR(SIG_INTERRUPT0) __attribute__((interrupt));
ISR(SIG_INTERRUPT0)
{	color++;
	if (color==3)
	{	color=0;
		spalte++;
	}
	ADMUX=(1<<REFS0)|(1<<REFS1)|(1<<ADLAR)|color;	// ADC0, 1,1V Ref
	sync=1;
}

ISR(SIG_INPUT_CAPTURE1)
{	DDRC=255;										// Black Level Clamping
	OCR1B=ICR1+spalte+32;							// Verzgerung seit fallender Flanke von HSync einstellen
	DDRC=0;
}



